import argparse
import os
import sys

from djangotools.cmdline.common.command import Command
from djangotools.config import load_app


class ArgsParser(argparse.ArgumentParser):

    default_settings_module = None

    def __init__(self, commands_dirs):
        super().__init__()
        self.commands_dirs = commands_dirs
        parser = self.add_subparsers(parser_class=argparse.ArgumentParser)
        self.add_argument("-a", "--app-dir",  help="Dossier de données du serveur")
        self.add_argument("-s", "--settings",  help="Module de settings à utiliser")

        for cmd in Command.load(commands_dirs):
            kwargs = {}
            if cmd.HELP: kwargs["help"] = cmd.HELP
            if cmd.ALIASES: kwargs["aliases"] = cmd.ALIASES
            p = parser.add_parser(cmd.NAME, **kwargs)
            for x in cmd.ARGUMENT or []:
                p.add_argument(*x.args, **x.kwargs)
            p.set_defaults(classe = cmd)

    def parse(self, args):
        ret = self.parse_args(args=args)
        settings = ret.settings or self.default_settings_module

        if settings:
            os.environ["DJANGO_SETTINGS_MODULE"] = settings
        else:
            print(f"Aucune module de setting n'a été passé merci d'utiliser la variable"
                  f" d'environnemnet DJANGO_SETTIGNS_MODULE ou le parmaètre --settings, -s", file =sys.stderr)
            exit(-1)

        load_app(settings, ret.app_dir)
        if getattr(ret, "classe", None) is None:
            self.print_help()
            exit(-1)
        ret.classe().run(ret)
