import threading
from pathlib import Path


class CommandData:

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class Argument:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

class ThreadWrapper(threading.Thread):

    def __init__(self, fct):
        super().__init__()
        self.fct = fct

    def run(self) -> None:
        return self.fct()


class Command:
    NAME = None
    HELP = None
    ALIASES = []

    ARGUMENT=None

    def __init__(self):
        pass

    def execute(self, data):
        return self.run(data)

    def run(self, data):
        raise NotImplementedError()

    @classmethod
    def thread_run(cls, data):
        cmd = cls()
        thread = ThreadWrapper(lambda x=data: cmd.run(x))
        thread.start()
        return thread

    @staticmethod
    def _load(root_module):
        ret = set()
        root = Path(root_module.__file__).parent
        queue = [root]
        root_module_str = root_module.__name__
        assert root_module_str != "__main__"
        while queue:
            current = queue.pop()
            for file in current.iterdir():
                if file.name == "__pycache__": continue
                if file.is_dir():
                    queue.append(file)
                    continue
                if file.is_file() and file.name.endswith(".py"):
                    module_name = root_module_str+"."+str(file.relative_to(root))[:-3].replace("/",".").replace(".py", "").replace("__init__", "")
                    if module_name[-1] == ".": module_name = module_name[:-1]
                    module = __import__(module_name, fromlist=['object'])
                    for x in dir(module):
                        data = getattr(module, x)
                        if isinstance(data, type) and issubclass(data, Command) and data != Command and data.NAME is not None:
                            ret.add(data)
        return ret

    @staticmethod
    def load(commands_dirs=None):
        if isinstance(commands_dirs, str): commands_dirs = [commands_dirs]
        from djangotools.cmdline import commands
        ret = set()
        commands_dirs = [commands]+list(commands_dirs or [])
        for commands_dir in commands_dirs:
            if isinstance(commands_dir, str):
                commands_dir = __import__(commands_dir, fromlist=['object'])
            ret |= Command._load(commands_dir)

        return ret





