import base64
import binascii
import tempfile
from pathlib import Path
from Crypto.Cipher import AES
import hashlib

from Crypto.Cipher import AES
from Crypto.Util import Counter
from Crypto import Random


class AsciiCypher:

    def __init__(self, key=None, file_key=None):
        key_data = key if isinstance(key, bytes) else (key.encode("utf-8") if key is not None else None)
        if key is None and file_key is None:
            raise ValueError("One of 'key' or 'file_key' must be provided")
        if key_data is None:
            if not Path(file_key).exists():
                raise FileNotFoundError('Le fichier de clé n\'existe pas')
            key_data = Path(file_key).read_bytes()
        hash = hashlib.sha256()
        hash.update(key_data)
        self.key = hash.digest()

    def do_encrypt(self, data) -> str:
        iv = Random.new().read(AES.block_size)
        iv_int = int(binascii.hexlify(iv), 16)
        ctr = Counter.new(AES.block_size * 8, initial_value=iv_int)
        aes = AES.new(self.key, AES.MODE_CTR, counter=ctr)
        ciphertext = aes.encrypt(data)
        iv = base64.b64encode(iv).decode()
        ciphertext = base64.b64encode(ciphertext).decode()
        return f"{iv}|{ciphertext}"

    def do_decrypt(self, data : str):
        if "|" not in data:
            raise ValueError("Bad value")
        iv, ciphertext = [base64.b64decode(x) for x in data.split('|')]
        iv_int = int.from_bytes(iv, byteorder='big')
        ctr = Counter.new(AES.block_size * 8, initial_value=iv_int)
        aes = AES.new(self.key, AES.MODE_CTR, counter=ctr)
        plaintext = aes.decrypt(ciphertext)
        return plaintext

    @classmethod
    def encrypt(cls, data, key=None, file=None):

        try:
            cypher = cls(key=key, file_key=file)
        except ValueError as e:
            print(e)
            return None
        if isinstance(data, str):
            data = data.encode("utf-8")
        return cypher.do_encrypt(data)

    @classmethod
    def decrypt(cls, data, key=None, file=None):

        try:
            cypher = cls(key=key, file_key=file)
        except Exception as e:
            print(e)
            return None

        return cypher.do_decrypt(data)

    @classmethod
    def decrypt_str(cls, data, key=None, file=None):
        x = cls.decrypt(data, key, file)
        if x:
            return x.decode("utf-8")
        return None


