from pathlib import Path

from djangotools.model import tools


class UrlPath:

    def __init__(self):
        self.liste = [self]

    def __truediv__(self, other):
        root = UrlPath()
        if isinstance(other, UrlPath):
            root.liste = self.liste + other.liste
        elif isinstance(other, str):
            root.liste = self.liste + [ConstPath(other)]

        return root

    def iter_models(self):
        for x in self.liste:
            if isinstance(x, (ModelPath, AttributePath)):
                yield x
        return

    def resolve(self):
        return [self]

    def get_wrapper(self, function):
        return

    def execute(self, manager):
        root = Path(".")
        for p in self.liste:
            x =  p.resolve(manager)
            if x is not None:
                root = root / x
        return root

    def copy(self):
        root = UrlPath()
        root.liste = list(self.liste)
        return root

    def get_path(self):
        liste = []
        for x in self.liste:
            liste.extend(x.resolve())
        self.liste=liste
        parts = [x.part for x in self.liste]
        return "/".join([ x for x in parts if x])

    @property
    def part(self):
        raise NotImplementedError()



class ConstPath(UrlPath):

    def __init__(self, string):
        super().__init__()
        self.string = string

    @property
    def part(self):
        return self.string

    def __repr__(self):
        return self.string

class AttributePath(UrlPath):

    def __init__(self, model, attr):
        super().__init__()
        self.model = model
        self.attr = attr
        self.model_name = self.model if isinstance(self.model, str) else model.__name__
        self.key = f"{self.model_name.lower()}_{self.attr}"
        self.forward = None

    @property
    def part(self):
        return f"<str:{self.key}>"


class ModelPath(UrlPath):

    def __init__(self, model):
        super().__init__()
        self.model = model

    def __getitem__(self, item):
        return self / AttributePath(self.model, item)

    @property
    def part(self):
        return None

class DeferredModelPath(UrlPath):
    def __init__(self, model):
        super().__init__()
        self.model = model
        self.attr = None


    def resolve(self):
        model = tools.get_model(self.model)
        root = model._path_.resolve() if model._path_ else PointPath()

        attribute = AttributePath(model, model._key_)
        attribute.forward = self.attr
        ret =  ( root / ConstPath(model.__name__.lower()) / attribute).liste
        return ret

    def __getitem__(self, item):
        self.attr = item
        return self



class PointPath(UrlPath):
    @property
    def part(self):
        return None
