from django.contrib.auth.models import User
from django.db.models import Manager, Model, QuerySet
from django.db.models.base import ModelBase

from djangotools.common import response
from djangotools.common.path import PointPath, ModelPath, AttributePath
from djangotools.model.serializedmodel import SerializableModel


class Route:
    GET = "GET"
    POST = "POST"
    DELETE = "DELETE"
    PUT = "PUT"
    method  = {GET, POST, DELETE, PUT}
    def __init__(self, route=None,  method=None):
        self.method = method if method is not None else self.method
        if isinstance(method, str): self.method = {method}
        self.path = route or PointPath()

    def __call__(self, *args, **kwargs):
        return RegisteredRoute(self, args[0])

    def __truediv__(self, other):
        if isinstance(other, Route):
            return Route(self.path / other.path, method=other.method)
        if isinstance(other, RegisteredRoute):
            return RegisteredRoute(self / other.route, other.callback)
        raise Exception()



class RegisteredRoute:
    _default_user_id_ = 1
    def __init__(self, path : Route, callback, is_manager=False, is_method=True, **options):
        self.route = path if path is not None else Route()
        self.callback = callback
        self.is_manager = is_manager
        self.is_method = is_method
        self.options = options

    def __call__(self, req, *args, **kwargs):
        obj = None
        if self._default_user_id_:
            req.user = User.objects.get(id=self._default_user_id_)


        for part in self.route.path.iter_models():
            if obj is None:
                obj = part.model.objects

            if isinstance(part, ModelPath):
                if not isinstance(obj, Manager):
                    obj = part.model.objects
            elif isinstance(part, AttributePath):
                obj = obj.filter(**{part.attr: kwargs[part.key]})
                if len(obj) > 2:
                    return response.serv_json_bad_request(f"Trop de résultats pour {part.model_name}{part.attr} = {kwargs[part.key]}")
                elif len(obj) == 0:
                    return response.serv_json_not_found(f"Aucun résultat pour {part.model_name}.{part.attr} = {kwargs[part.key]}")
                else:
                    kwargs.pop(part.key)
                    obj = obj.first()
                    if part.forward:
                        obj = getattr(obj, part.forward)
            else:
                raise Exception()


        if self.is_manager:
            if isinstance(obj, Model):
                obj = obj.__class__.objects
            elif isinstance(obj, ModelBase):
                obj = obj.objects
        if self.is_method:
            ret = self.callback(obj, req, *args, **kwargs)
        else:
            ret = self.callback(req, obj, *args, **kwargs)

        if isinstance(ret, QuerySet):
            return [x.serialize() for x in ret]
        if isinstance(ret, SerializableModel):
            return ret.serialize()
        if isinstance(ret, list):
            ret = [x.serialize() if isinstance(x, SerializableModel) else x for x in ret]
        return ret



    def copy(self):
        return RegisteredRoute(self.route, self.callback, is_manager=self.is_manager,
                               is_method=self.is_method)

class Get(Route):
    method = {Route.GET}
    
class Post(Route):
    method = {Route.POST}

class Put(Route):
    method = {Route.PUT}

class Delete(Route):
    method = {Route.DELETE}
