import datetime
from collections.abc import Iterable
from typing import Callable

from django.db import models



class Manager(models.Manager):
    FIELD_CREATE_ITEM=()
    def create_from_json(self, **kwargs):
        return self.create(**kwargs)




class Serializer:
    def __init__(self, attr_name=None, from_json=None, to_json=None, validate=None, on_update=None):
        self.attr_name = attr_name
        self._from_json=from_json
        self._to_json =to_json
        self._validate = validate
        self._on_update = on_update

    def notify_update(self, obj, attrname):
        if not self._on_update: return None
        if isinstance(self._on_update, str):
            getattr(obj, self._on_update)(attrname)
        else:
            self._on_update(obj, attrname)

    def serialize(self, x, obj, context):
        if self._to_json:
            return self._to_json(x)
        if x is None:
            return None
        if isinstance(x, (int, str, float, bytes, str)):
            return x
        if isinstance(x, datetime.date):
            return f"{str(x.year).zfill(4)}/{str(x.month).zfill(2)}/{str(x.day).zfill(2)}"
        if isinstance(x, datetime.datetime):
            return f"{str(x.year).zfill(4)}/{str(x.month).zfill(2)}/{str(x.day).zfill(2)} {str(x.hour).zfill(2)}:{str(x.minute).zfill(2)}:{str(x.second).zfill(2)}.{str(x.microsecond).zfill(6)}"


    def validate(self, x):
        if self._validate: self._validate(x)

    def get_value(self, x, old_value):
        self.validate(x)
        return self._from_json(x) if self._to_json else x


class DateSerializer(Serializer):
    def get_value(self, x, old_value):
        x = "".join([a for  a in x.replace("-","/") if a in '0123456789/'])
        return datetime.datetime.strptime(x, "%d/%m/%Y").date()
class ManyToOneSerializer(Serializer):

    def serialize(self, x, obj, context):
        return [a.serialize() for a in x.all() ]

class _BypassProperty:
    pass

BypassProperty = _BypassProperty()

class FunctionSerilizer(Serializer):
    def __init__(self, property, name=None):
        name = name or property
        super().__init__(name)
        self._property_name = property

    def serialize(self, x, obj, context):
        return getattr(obj, self._property_name).fct(obj, context=context)


class PropertySerializer(Serializer):
    def serialize(self, x, obj, context):
        return x


def compare_dict(model_dict, form_dict):
    for k, v in model_dict.items():
        fv = form_dict.get(k)
        if v is not None and v != "":
            if not fv or fv!=v:
                return False
        elif fv is not None and fv != "":
            if not v or fv!=v:
                return False

    return True

def compare_list_dict(model, form):
    model=list(model)
    if len(model) != len(form):
        return False

    for i in range(len(model)):
        if not compare_dict(model[i], form[i]): return False
    return True

class SerializeField:
    def __init__(self, fct, *args, **kwargs):
        self.args =args
        self.kwargs = kwargs
        self.fct = fct

class serialize:

    def __init__(self, *args, **kwargs):
        if(args and len(args) and isinstance(args[0], Callable)):
            self._is_simple = True
            self.fct = args[0]
            self.args =[]
            self.kwargs = {}
            kwargs["name"] = self.fct.__name__
        else:
            self.args =args
            self.kwargs = kwargs
            if args and len(args)==1: kwargs["name"] = args[0]
            self.fct = None

    def __call__(self, fct):
        return SerializeField(fct, *self.args, **self.kwargs)

class SerializableModel:
    class Empty:
        pass

    empty=Empty()
    _REGISTERED={}
    serialize_fields = {}
    bypass_fields = {}

    @classmethod
    def register(cls):
        SerializableModel._REGISTERED[cls.__name__.lower()]=cls

    @staticmethod
    def resolve(name):
        return SerializableModel._REGISTERED.get(name.lower())

    def _get_decorated_fields(self):
        fields = {k: getattr(self, k) for k in dir(self) if k!="objects" and  isinstance(getattr(self.__class__,k, None), (SerializeField, serialize))}
        out = {}
        for name, fct in fields.items():
            if(fct.fct.__class__.__name__ == "property"):
                raise ValueError("Cant do that on preperty....")
                out[name] = PropertySerilizer(name)
            if(fct.fct.__class__.__name__ == "function"):
                out[name] = FunctionSerilizer(name, fct.kwargs.get("name") or name)
        return out

    def get_fields(self):
        ser = {field.attname: Serializer() for field in self._meta.fields if field.attname not in self.bypass_fields}
        ser.update(self.serialize_fields if hasattr(self, "serialize_fields") else {})
        ser.update(self._get_decorated_fields())
        return ser


    def serialize(self, **context):
        x=self.get_fields()
        ret = {
            v.attr_name or k: v.serialize(getattr(self, k), self, context) for k, v in x.items()
        }
        return {k:v for k, v in ret.items() if v != BypassProperty}

    def update(self, x):
        fields=self.get_fields()
        reverse = {v.attr_name: k for k, v in fields.items() if v.attr_name}
        for k, v in x.items():
            serial = fields.get(k)
            if serial:
                dictname = attrname = k
            elif k in reverse:
                dictname = k
                attrname = reverse[k]
            else:
                continue
            if hasattr(self, attrname):
                old_value = getattr(self, attrname)
                if dictname in fields:
                    attr = getattr(self, attrname)
                    if attr and hasattr(attr, "model"):
                        _old_value = [x.json for x in attr.all()]
                        values = serial.get_value(v, old_value)
                        if not compare_list_dict(_old_value, values):
                            attr.all().delete()
                            for x in values:
                                attr.model.objects.create_from_parent(parent=self, **x)
                            serial.notify_update(self, attrname)
                    else:
                        serial = fields.get(dictname)
                        new_value = serial.get_value(v, old_value)
                        if old_value != new_value:
                            setattr(self, attrname, new_value)
                            serial.notify_update(self, attrname)



