from django.contrib.staticfiles.management.commands.runserver import Command as RunServer
from django.core.management.commands.migrate import Command as MigrateCommand

from djangotools.config import load_app


class Server(RunServer):
    default_options = {
        'verbosity': 1,
        'settings': None,
        'pythonpath': None,
        'traceback': True,
        'no_color': False,
        'force_color': True,
        'addrport': "localhost:8000",
        'use_ipv6': False,
        'use_threading': False,
        'use_reloader': False,
        'skip_checks': False,
        'use_static_handler': True,
        'insecure_serving': True
    }


    def handle(self, **options):
        for k, v in self.default_options.items():
            options.setdefault(k, v)

        super().handle(**options)


class Migrate(MigrateCommand):

    default_options = {
        'verbosity': 1,
        "interactive" : False,
        "skip_checks" : False,
        "database" : "default",
        "prune" : False,
        "check_unapplied" : False,
        "run_syncdb" : False,
        "noinput" : True,
        "fake" : False,
        "fake_initial" : False,
        "plan" : False,
        "app_label" : None,

    }
    def handle(self, **options):
        for k, v in self.default_options.items():
            options.setdefault(k, v)

        super().handle(**options)

if __name__ == "__main__":
    cmd = Server()
    load_app("banque.banque.settings")
    cmd.handle()

