from collections import defaultdict
from functools import partial

from django.db.models.base import ModelBase
from django.http import HttpRequest, HttpResponse
from django.urls import path, URLPattern

from djangotools.common import response
from djangotools.common.route import RegisteredRoute, Route
from djangotools import  DjangoTools
from djangotools.model.serializedmodel import SerializableModel
from djangotools.model.tools import iter_models

from django.urls import path
class _Router:

    class _Route(Route):
        def __call__(self, fct, **options):
            rr =RegisteredRoute(self, fct, is_method=False, **options)
            Router.add(rr)
            return rr

    def __init__(self):
        self._models = {}
        for  model in iter_models(DjangoTools.app):
            if isinstance(model, type) and issubclass(model, SerializableModel):
                self._models[model.__name__] = model
        self.routes = defaultdict(dict)
        self._raw_pathes = []

    def route(self, route=None, method=None, **options):
        return self._Route(route, method, **options)

    def get(self, path, **options):
        return self.route(path, Route.GET, **options)

    def post(self, path, **options):
        return self.route(path, Route.POST, **options)

    def put(self, path, **options):
        return self.route(path, Route.PUT, **options)


    def delete(self, path, **options):
        return self.route(path, Route.DELETE, **options)


    def __iadd__(self, other):
        self.add(other)
        return self

    def add(self, registerd_route, override=True):
        print("add", registerd_route)
        if isinstance(registerd_route, RegisteredRoute):
            for method in registerd_route.route.method:
                current_path = registerd_route.route.path.get_path()
                if method in self.routes[current_path]:
                    if not override:
                        raise ValueError(f"La route {current_path} est déja défini avec {method}")
                self.routes[current_path][method] = registerd_route
        elif isinstance(registerd_route, (list, tuple)):
            [self.add(x) for x in registerd_route]
        elif isinstance(registerd_route, (partial, URLPattern)):
            self._raw_pathes.append(registerd_route)
        else:
            print(registerd_route)
            raise Exception()

    def resolve_model(self, model):
        if isinstance(model, ModelBase): return model
        return self._models[model]

    def get_pathes(self):
        pathes = []
        for route, methods in self.routes.items():
            def wrapper(req: HttpRequest, *args, methods=methods, **kwargs):
                method = req.method
                if method not in methods:
                    return response.serv_json_not_found(f"Method {method} on {req.path}")
                ret = methods[method](req, *args, **kwargs)
                if isinstance(ret, HttpResponse):
                    return ret
                return response.serv_json_ok(ret)

            pathes.append(path(route, wrapper))
        return pathes + self._raw_pathes




Router = _Router()